<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

declare(strict_types=1);

use Migrations\AbstractMigration;

/**
 * Class VersionFour
 *
 * Upgrade Version 3 schema to Version 4
 *
 * Created:
 * oitc bake migration VersionFour
 *
 * Run migration:
 * oitc migrations migrate
 */
class VersionFour extends AbstractMigration {

    /**
     * Whether the tables created in this migration
     * should auto-create an `id` field or not
     *
     * This option is global for all tables created in the migration file.
     * If you set it to false, you have to manually add the primary keys for your
     * tables using the Migrations\Table::addPrimaryKey() method
     *
     * @var bool
     */
    public $autoId = false;

    /**
     * Change Method.
     *
     * More information on this method is available here:
     * https://book.cakephp.org/phinx/0/en/migrations.html#the-change-method
     * @return void
     */
    public function change(): void {
        $this->table('acos')
            ->addColumn('id', 'integer', [
                'autoIncrement' => true,
                'default'       => null,
                'limit'         => 10,
                'null'          => false,
            ])
            ->addPrimaryKey(['id'])
            ->addColumn('parent_id', 'integer', [
                'default' => null,
                'limit'   => 10,
                'null'    => true,
            ])
            ->addColumn('model', 'string', [
                'default' => null,
                'limit'   => 255,
                'null'    => true,
            ])
            ->addColumn('foreign_key', 'integer', [
                'default' => null,
                'limit'   => 10,
                'null'    => true,
            ])
            ->addColumn('alias', 'string', [
                'default' => null,
                'limit'   => 255,
                'null'    => true,
            ])
            ->addColumn('lft', 'integer', [
                'default' => null,
                'limit'   => 10,
                'null'    => true,
            ])
            ->addColumn('rght', 'integer', [
                'default' => null,
                'limit'   => 10,
                'null'    => true,
            ])
            ->create();


        $this->table('aros')
            ->addColumn('id', 'integer', [
                'autoIncrement' => true,
                'default'       => null,
                'limit'         => 10,
                'null'          => false,
            ])
            ->addPrimaryKey(['id'])
            ->addColumn('parent_id', 'integer', [
                'default' => null,
                'limit'   => 10,
                'null'    => true,
            ])
            ->addColumn('model', 'string', [
                'default' => null,
                'limit'   => 255,
                'null'    => true,
            ])
            ->addColumn('foreign_key', 'integer', [
                'default' => null,
                'limit'   => 10,
                'null'    => true,
            ])
            ->addColumn('alias', 'string', [
                'default' => null,
                'limit'   => 255,
                'null'    => true,
            ])
            ->addColumn('lft', 'integer', [
                'default' => null,
                'limit'   => 10,
                'null'    => true,
            ])
            ->addColumn('rght', 'integer', [
                'default' => null,
                'limit'   => 10,
                'null'    => true,
            ])
            ->create();


        $this->table('aros_acos')
            ->addColumn('id', 'integer', [
                'autoIncrement' => true,
                'default'       => null,
                'limit'         => 10,
                'null'          => false,
            ])
            ->addPrimaryKey(['id'])
            ->addColumn('aro_id', 'integer', [
                'default' => null,
                'limit'   => 10,
                'null'    => false,
            ])
            ->addColumn('aco_id', 'integer', [
                'default' => null,
                'limit'   => 10,
                'null'    => false,
            ])
            ->addColumn('_create', 'string', [
                'default' => '0',
                'limit'   => 2,
                'null'    => false,
            ])
            ->addColumn('_read', 'string', [
                'default' => '0',
                'limit'   => 2,
                'null'    => false,
            ])
            ->addColumn('_update', 'string', [
                'default' => '0',
                'limit'   => 2,
                'null'    => false,
            ])
            ->addColumn('_delete', 'string', [
                'default' => '0',
                'limit'   => 2,
                'null'    => false,
            ])
            ->addIndex(
                [
                    'aro_id',
                    'aco_id',
                ],
                ['unique' => true]
            )
            ->create();

        $this->table('apikeys')
            ->addColumn('id', 'integer', [
                'autoIncrement' => true,
                'default'       => null,
                'limit'         => 11,
                'null'          => false,
            ])
            ->addPrimaryKey(['id'])
            ->addColumn('user_id', 'integer', [
                'default' => null,
                'limit'   => 11,
                'null'    => false,
            ])
            ->addColumn('apikey', 'string', [
                'default' => null,
                'limit'   => 255,
                'null'    => false,
            ])
            ->addColumn('description', 'string', [
                'default' => null,
                'limit'   => 255,
                'null'    => true,
            ])
            ->addIndex(
                [
                    'apikey',
                    'user_id',
                ]
            )
            ->create();

        $this->table('users')
            ->addColumn('usergroup_id', 'integer', [
                'default' => '1',
                'null'    => false,
                'after'   => 'password'
            ])
            ->addColumn('timezone', 'string', [
                'default' => 'Europe/Berlin',
                'limit'   => 100,
                'null'    => true,
                'after'   => 'lastname'
            ])
            ->addColumn('dateformat', 'string', [
                'default' => 'H:i:s - d.m.Y',
                'limit'   => 100,
                'null'    => true,
                'after'   => 'lastname'
            ])
            ->addColumn('image', 'string', [
                'default' => null,
                'limit'   => 100,
                'null'    => true,
                'after'   => 'lastname'
            ])
            ->addColumn('showstatsinmenu', 'boolean', [
                'default' => 0,
                'limit'   => null,
                'null'    => false,
                'after'   => 'lastname'
            ])
            ->addColumn('paginatorlength', 'integer', [
                'default' => '25',
                'limit'   => 4,
                'null'    => false,
                'after'   => 'lastname'
            ])
            ->update();


        $this->table('usergroups')
            ->addColumn('id', 'integer', [
                'autoIncrement' => true,
                'default'       => null,
                'limit'         => 11,
                'null'          => false,
            ])
            ->addPrimaryKey(['id'])
            ->addColumn('name', 'string', [
                'default' => null,
                'limit'   => 255,
                'null'    => false,
            ])
            ->addColumn('description', 'string', [
                'default' => null,
                'limit'   => 255,
                'null'    => true,
            ])
            ->addColumn('created', 'datetime', [
                'default' => null,
                'limit'   => null,
                'null'    => false,
            ])
            ->addColumn('modified', 'datetime', [
                'default' => null,
                'limit'   => null,
                'null'    => false,
            ])
            ->create();
    }
}
