<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

declare(strict_types=1);

namespace MapModule\Model\Table;

use Cake\Datasource\EntityInterface;
use Cake\Datasource\RepositoryInterface;
use Cake\ORM\Association\BelongsTo;
use Cake\ORM\Behavior\TimestampBehavior;
use Cake\ORM\RulesChecker;
use Cake\ORM\Table;
use Cake\Validation\Validator;
use MapModule\Model\Entity\Mapgadget;

/**
 * Mapgadgets Model
 *
 * @property MapsTable&BelongsTo $Maps
 *
 * @method Mapgadget get($primaryKey, $options = [])
 * @method Mapgadget newEntity($data = null, array $options = [])
 * @method Mapgadget[] newEntities(array $data, array $options = [])
 * @method Mapgadget|false save(EntityInterface $entity, $options = [])
 * @method Mapgadget saveOrFail(EntityInterface $entity, $options = [])
 * @method Mapgadget patchEntity(EntityInterface $entity, array $data, array $options = [])
 * @method Mapgadget[] patchEntities($entities, array $data, array $options = [])
 * @method Mapgadget findOrCreate($search, callable $callback = null, $options = [])
 *
 * @mixin TimestampBehavior
 */
class MapgadgetsTable extends Table {
    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config): void {
        parent::initialize($config);

        $this->setTable('mapgadgets');
        $this->setDisplayField('id');
        $this->setPrimaryKey('id');

        $this->addBehavior('Timestamp');

        $this->belongsTo('Maps', [
            'foreignKey' => 'map_id',
            'joinType'   => 'INNER',
            'className'  => 'MapModule.Maps',
        ]);

        $this->hasMany('Mappings');
    }

    public function bindCoreAssociations(RepositoryInterface $coreTable) {
        switch ($coreTable->getAlias()) {
            case 'Services':
                $coreTable->hasMany('Mapgadgets', [
                    'className'  => 'MapModule.Mapgadgets',
                    'dependent'  => true,
                    'foreignKey' => 'object_id',
                    'joinType'   => 'INNER',
                    'conditions' => [
                        'type' => 'service'
                    ]
                ]);
                break;
        }
    }

    /**
     * Default validation rules.
     *
     * @param Validator $validator Validator instance.
     * @return Validator
     */
    public function validationDefault(Validator $validator): Validator {
        return $validator;
    }

    /**
     * Returns a rules checker object that will be used for validating
     * application integrity.
     *
     * @param RulesChecker $rules The rules object to be modified.
     * @return RulesChecker
     */
    public function buildRules(RulesChecker $rules): RulesChecker {
        $rules->add($rules->existsIn(['map_id'], 'Maps'));

        return $rules;
    }

    /**
     * @param $id
     * @return bool
     */
    public function existsById($id) {
        return $this->exists(['Mapgadgets.id' => $id]);
    }
}
