<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.
declare(strict_types=1);

namespace App\Controller;

use App\Model\Entity\Hostcheck;
use App\Model\Table\HostchecksTable;
use Cake\ORM\TableRegistry;
use itnovum\openITCOCKPIT\Core\AngularJS\Request\HostchecksControllerRequest;
use itnovum\openITCOCKPIT\Core\HostcheckConditions;
use itnovum\openITCOCKPIT\Core\ValueObjects\User;
use itnovum\openITCOCKPIT\Core\Views\UserTime;
use itnovum\openITCOCKPIT\Database\PaginateOMat;

/**
 * Class HostchecksController
 * @package App\Controller
 */
class HostchecksController extends AppController {

    public function index() {
        if (!$this->isApiRequest()) {
            //Only ship html template
            return;
        }

        $session = $this->request->getSession();
        $session->close();

        $HostchecksControllerRequest = new HostchecksControllerRequest($this->request);
        $PaginateOMat = new PaginateOMat($this, $this->isScrollRequest(), $HostchecksControllerRequest->getPage());

        //Process conditions
        $Conditions = new HostcheckConditions();
        $Conditions->setFrom($HostchecksControllerRequest->getFrom());
        $Conditions->setTo($HostchecksControllerRequest->getTo());
        $Conditions->setStates($HostchecksControllerRequest->getHostStates());
        $Conditions->setStateTypes($HostchecksControllerRequest->getHostStateTypes());
        $Conditions->setOrder($HostchecksControllerRequest->getOrderForPaginator('Hostchecks.start_time', 'desc'));
        $Conditions->setConditions($HostchecksControllerRequest->getIndexFilters());

        $UserTime = UserTime::fromUser($this->getUser());

        /** @var HostchecksTable $HostchecksTable */
        $HostchecksTable = TableRegistry::getTableLocator()->get('Hostchecks');

        $all_hostchecks = $HostchecksTable->getHostchecks($Conditions, $PaginateOMat);

        foreach ($all_hostchecks as $index => $hostcheck) {
            /** @var Hostcheck $hostcheck */
            $hostcheck->setUserTime($UserTime);
        }

        $this->set('all_hostchecks', $all_hostchecks);
        $toJson = ['all_hostchecks', 'paging'];
        if ($this->isScrollRequest()) {
            $toJson = ['all_hostchecks', 'scroll'];
        }
        $this->viewBuilder()->setOption('serialize', $toJson);
    }

}
