<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

namespace App\Identifier;

use App\Model\Table\UsersTable;
use ArrayAccess;
use Authentication\Identifier\AbstractIdentifier;
use Authentication\Identifier\IdentifierInterface;
use Cake\Datasource\EntityInterface;
use Cake\ORM\TableRegistry;

class ApikeyIdentifier extends AbstractIdentifier implements IdentifierInterface {

    /**
     * Identifies an user or service by the passed credentials
     *
     * @param array $credentials Authentication credentials
     * @return array|ArrayAccess|EntityInterface|null
     */
    public function identify(array $credentials) {
        if (isset($credentials['apikey']) && $credentials['apikey'] !== null) {
            $identity = $this->_findIdentity($credentials['apikey']);

            return $identity;
        }

        return null;
    }

    /**
     * Find a user record using the apikey/identifier provided.
     *
     * @param string $apikey
     * @return array|EntityInterface|null
     */
    protected function _findIdentity(string $apikey) {
        /** @var UsersTable $UsersTable */
        $UsersTable = TableRegistry::getTableLocator()->get('Users');

        if (isset($apikey) && strlen($apikey) > 1) {
            return $UsersTable->getUserByApikeyForLogin($apikey);
        }

        return null;
    }
}
