<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

namespace App\Model\Entity;

use Cake\ORM\Entity;
use itnovum\openITCOCKPIT\Core\Views\BBCodeParser;
use itnovum\openITCOCKPIT\Core\Views\UserTime;

/**
 * Hoststatus Entity
 *
 * @property string $hostname
 * @property int $status_update_time
 * @property string $output
 * @property string $long_output
 * @property string $perfdata
 * @property int $current_state
 * @property bool $current_check_attempt
 * @property bool $max_check_attempts
 * @property int $last_check
 * @property int $next_check
 * @property bool $is_passive_check
 * @property int $last_state_change
 * @property int $last_hard_state_change
 * @property bool $last_hard_state
 * @property bool $is_hardstate
 * @property int $last_notification
 * @property int $next_notification
 * @property bool $notifications_enabled
 * @property bool $problem_has_been_acknowledged
 * @property bool $acknowledgement_type
 * @property bool $passive_checks_enabled
 * @property bool $active_checks_enabled
 * @property bool $event_handler_enabled
 * @property bool $flap_detection_enabled
 * @property bool $is_flapping
 * @property float $latency
 * @property float $execution_time
 * @property bool $scheduled_downtime_depth
 * @property bool $process_performance_data
 * @property bool $obsess_over_host
 * @property int $normal_check_interval
 * @property int $retry_check_interval
 * @property string $check_timeperiod
 * @property string $node_name
 * @property int $last_time_up
 * @property int $last_time_down
 * @property int $last_time_unreachable
 * @property int $current_notification_number
 * @property float $percent_state_change
 * @property string $event_handler
 * @property string $check_command
 */
class Hoststatus extends Entity {

    /**
     * @var UserTime|null
     */
    private $UserTime = null;

    /**
     * @var BBCodeParser|null
     */
    private $BBCodeParser = null;

    /**
     * Fields that can be mass assigned using newEntity() or patchEntity().
     *
     * Note that when '*' is set to true, this allows all unspecified fields to
     * be mass assigned. For security purposes, it is advised to set '*' to false
     * (or remove it), and explicitly make individual fields accessible as needed.
     *
     * @var array
     */
    protected $_accessible = [
        '*'        => true,
        'hostname' => false
    ];

    /**
     * @param UserTime $UserTime
     */
    public function setUserTime(UserTime $UserTime) {
        $this->UserTime = $UserTime;
    }

    public function setBBCodeParser(BBCodeParser $BBCodeParser) {
        $this->BBCodeParser = $BBCodeParser;
    }

    /**
     * @return array
     * @throws \Exception
     */
    public function jsonSerialize(): array {
        if ($this->UserTime === null && $this->BBCodeParser === null) {
            return parent::jsonSerialize();
        }

        $data = parent::jsonSerialize();

        if ($this->UserTime) {
            $fields = [
                'status_update_time',
                'last_check',
                'next_check',
                'last_state_change',
                'last_hard_state_change',
                'last_time_up',
                'last_time_down',
                'last_time_unreachable'
            ];

            foreach ($fields as $field) {
                if (isset($data[$field])) {
                    $data[$field] = $this->UserTime->format($data[$field]);
                }
            }

            $fieldsAgoInWords = [
                'last_check',
                'next_check'
            ];

            foreach ($fieldsAgoInWords as $field) {
                if (isset($data[$field])) {
                    $data[$field . '_in_words'] = $this->UserTime->timeAgoInWords($this->{$field});
                }
            }

            $humanFields = [
                'last_state_change',
                'last_hard_state_change'
            ];

            foreach ($humanFields as $field) {
                if (isset($data[$field])) {
                    $data[$field . '_for_humans'] = $this->UserTime->secondsInHumanShort(time() - $this->{$field});
                }
            }

            $humanFields = [
                'normal_check_interval',
                'retry_check_interval'
            ];

            foreach ($humanFields as $field) {
                if (isset($data[$field])) {
                    $data[$field . '_for_humans'] = $this->UserTime->secondsInHumanShort($this->{$field});
                }
            }
        }

        if ($this->BBCodeParser) {
            $bbFields = [
                'output',
                'long_output'
            ];

            foreach ($bbFields as $field) {
                if (isset($data[$field])) {
                    $data[$field . '_html'] = $this->BBCodeParser->nagiosNl2br($this->BBCodeParser->asHtml($this->{$field}, true));
                }
            }
        }

        return $data;
    }
}
