<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

namespace App\Model\Table;

use App\Filter\ServicecheckFilter;
use App\Lib\Traits\PaginationAndScrollIndexTrait;
use App\Model\Entity\Servicecheck;
use Cake\Datasource\EntityInterface;
use Cake\ORM\RulesChecker;
use Cake\ORM\Table;
use Cake\Validation\Validator;
use itnovum\openITCOCKPIT\Database\PaginateOMat;

/**
 * Servicechecks Model
 *
 * @method Servicecheck get($primaryKey, $options = [])
 * @method Servicecheck newEntity($data = null, array $options = [])
 * @method Servicecheck[] newEntities(array $data, array $options = [])
 * @method Servicecheck|bool save(EntityInterface $entity, $options = [])
 * @method Servicecheck patchEntity(EntityInterface $entity, array $data, array $options = [])
 * @method Servicecheck[] patchEntities($entities, array $data, array $options = [])
 * @method Servicecheck findOrCreate($search, callable $callback = null, $options = [])
 */
class ServicechecksTable extends Table {
    use PaginationAndScrollIndexTrait;

    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config): void {
        parent::initialize($config);

        $this->setTable('statusengine_servicechecks');
    }

    /**
     * Default validation rules.
     *
     * @param \Cake\Validation\Validator $validator Validator instance.
     * @return \Cake\Validation\Validator
     */
    public function validationDefault(Validator $validator): Validator {
        //Readonly table
        return $validator;
    }

    /**
     * Returns a rules checker object that will be used for validating
     * application integrity.
     *
     * @param \Cake\ORM\RulesChecker $rules The rules object to be modified.
     * @return \Cake\ORM\RulesChecker
     */
    public function buildRules(RulesChecker $rules): RulesChecker {
        //Readonly table
        return $rules;
    }

    public function getServiceChecks(ServicecheckFilter $ServicecheckFilter, ?PaginateOMat $PaginateOMat = null) {
        $query = $this->find()
            ->where($ServicecheckFilter->indexFilter())
            ->order($ServicecheckFilter->getOrderForPaginator('Servicechecks.start_time', 'desc'));

        if ($PaginateOMat === null) {
            //Just execute query
            $result = $query->toArray();
        } else {
            if ($PaginateOMat->useScroll()) {
                $result = $this->scrollCake4($query, $PaginateOMat->getHandler());
            } else {
                $result = $this->paginateCake4($query, $PaginateOMat->getHandler());
            }
        }

        return $result;
    }
}
