<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

namespace App\View\Helper;

class ButtonGroupHelper implements ButtonGroupHelperInterface {

    private $groupElements = [];
    private $buttonGroupAriaLabel = '';

    public function __construct(string $buttonGroupAriaLabel = '') {
        $this->buttonGroupAriaLabel = $buttonGroupAriaLabel;
    }

    public function getHtml(): string {
        return $this->createButtonGroupWithContent();
    }

    private function createButtonGroupWithContent(): string {
        $html = $this->buttonGroupOpens();

        $html .= $this->iterateOverManuallyAddedElements();

        $html .= $this->buttonGroupCloses();

        return $html;
    }

    private function buttonGroupOpens(): string {
        return '<div class="btn-group mr-2" role="group" aria-label="' . $this->buttonGroupAriaLabel . '">';
    }

    private function iterateOverManuallyAddedElements(): string {
        $html = '';

        foreach ($this->groupElements as $element) {
            $html .= PHP_EOL . $this->indent() . $element;
        }

        return $html;
    }

    /**
     * @return string
     */
    private function indent(): string {
        return '    ';
    }

    private function buttonGroupCloses(): string {
        return PHP_EOL . '</div>';
    }

    public function addIconButton(string $iconCssSelector, string $dataOriginalTitle = ''): ButtonGroupHelperInterface {
        $this->groupElements[] = $this->getOpeningButtonGeneralPart('btn-default') . ' data-original-title="' . $dataOriginalTitle . '" data-placement="bottom" rel="tooltip" data-container="body"><i class="' . $iconCssSelector . '"></i></button>';

        return $this;
    }

    private function getOpeningButtonGeneralPart(string $classAttribute): string {
        return '<button class="btn ' . $classAttribute . ' "';
    }

    public function addButton(string $innerHtml, string $cssSelector = 'btn-default'): ButtonGroupHelperInterface {
        $this->groupElements[] = '<button class="btn ' . $cssSelector . '">' . $innerHtml . '</button>';

        return $this;
    }

    public function addButtonWithTooltip(string $innerHtml, string $cssSelector = 'btn-default', $dataOriginalTitle = ''): ButtonGroupHelperInterface {
        $this->groupElements[] = '<button class="btn ' . $cssSelector . '" data-original-title="' . $dataOriginalTitle . '" data-placement="bottom" rel="tooltip" data-container="body">' . $innerHtml . '</button>';

        return $this;
    }

    public function addButtonWithTooltipAndDisplayConditional(string $innerHtml, string $cssSelector = 'btn-default', string $dataOriginalTitle = '', string $conditional = ''): ButtonGroupHelperInterface {
        $this->groupElements[] = '<button class="btn ' . $cssSelector . '" data-original-title="' . $dataOriginalTitle . '" data-placement="bottom" rel="tooltip" data-container="body" ' . $conditional . '>' . $innerHtml . '</button>';

        return $this;
    }

    public function addRaw($html): ButtonGroupHelperInterface {
        $this->groupElements[] = $html;

        return $this;
    }

    public function addButtonWithTogglingMenu(string $iconCssSelector, string $dataOriginalTitle = '', string $htmlMenu = ''): ButtonGroupHelperInterface {
        $this->groupElements[] = '<a href="javascript:void(0);" class="btn btn-default" data-toggle="dropdown" data-original-title="' . $dataOriginalTitle . '"  data-placement="bottom" rel="tooltip"><i class="' . $iconCssSelector . '"></i></a>';
        $this->groupElements[] = $this->attachDropDownMenu($htmlMenu);

        return $this;
    }

    private function attachDropDownMenu(string $html): string {
        return $html;
    }

    public function addButtonWithTooltipAndSRef(string $innerHtml, string $cssSelector = 'btn-default', $dataOriginalTitle = '', $sRef = ''): ButtonGroupHelperInterface {
        $this->groupElements[] = '<button class="btn ' . $cssSelector . '" data-original-title="' . $dataOriginalTitle . '" data-placement="bottom" rel="tooltip" data-container="body" ui-sref="' . $sRef . '">' . $innerHtml . '</button>';

        return $this;
    }

    public function addIconButtonWithSRef(string $iconCssSelector, string $dataOriginalTitle = '', string $sRef = '', string $additionalHtmlAttributes = ''): ButtonGroupHelperInterface {
        $this->groupElements[] = '<button class="btn btn-default" data-original-title="' . $dataOriginalTitle . '" data-placement="bottom" rel="tooltip" data-container="body" ui-sref="' . $sRef . '" ' . $additionalHtmlAttributes . '><i class="' . $iconCssSelector . '"></i></button>';

        return $this;
    }

    public function addIconButtonWithHRef(string $iconCssSelector, string $dataOriginalTitle = '', string $hRef = ''): ButtonGroupHelperInterface {
        $this->groupElements[] = $this->openingButtonHtmlTag('btn-default', 'data-original-title="' . $dataOriginalTitle . '" data-placement="bottom" rel="tooltip" data-container="body"') . '<a href="' . $hRef . '"><i class="' . $iconCssSelector . '"></i></a></button>';

        return $this;
    }

    private function openingButtonHtmlTag(string $additionalClassProperties, string $htmlAttributes): string {
        return '<button class="btn ' . $additionalClassProperties . ' " ' . $htmlAttributes . '>';
    }

    public function addIconButtonWithHRefFixed(string $iconCssSelector, string $dataOriginalTitle = '', string $hRef = ''): ButtonGroupHelperInterface {
        $this->groupElements[] = '<a class="btn btn-default" data-original-title="' . $dataOriginalTitle . '" data-placement="bottom" rel="tooltip" data-container="body" href="' . $hRef . '"><i class="' . $iconCssSelector . '"></i></a>';

        return $this;
    }
}
