<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

declare(strict_types=1);

namespace itnovum\openITCOCKPIT\Core\Menu;


class MenuCategory {

    /**
     * @var string
     */
    private $name;

    /**
     * @var string
     */
    private $alias;

    /**
     * @var int
     */
    private $order;

    /**
     * @var string
     */
    private $icon;

    /**
     * @var array
     */
    private $links = [];

    /**
     * MenuCategory constructor.
     * @param string $name
     * @param int $order
     * @param string $icon
     * @return $this
     */
    public function __construct(string $name, string $alias = '', int $order = 0, string $icon = '') {
        $this->name = $name;
        $this->alias = $alias;
        $this->order = $order;
        $this->icon = $icon;
        return $this;
    }

    /**
     * @param MenuLink $MenuLink
     * @return $this
     */
    public function addLink(MenuLink $MenuLink) {
        $links = $this->links;
        $links[] = $MenuLink;

        $this->links = [];

        $indexesToOrder = [];
        foreach ($links as $index => $link) {
            /** @var MenuLink $link */
            $indexesToOrder[$index] = $link->getOrder();
        }

        asort($indexesToOrder);

        foreach ($indexesToOrder as $index => $orderNumber) {
            $this->links[] = $links[$index];
        }

        return $this;
    }

    /**
     * @param int $index
     * @return bool
     */
    public function removeMenuLinkByIndex(int $index): bool {
        if (isset($this->links[$index])) {
            unset($this->links[$index]);
            return true;
        }

        return false;
    }

    /**
     * @return string
     */
    public function getName(): string {
        return $this->name;
    }

    /**
     * @return int
     */
    public function getOrder(): int {
        return $this->order;
    }

    /**
     * @return string
     */
    public function getIcon(): string {
        return $this->icon;
    }

    /**
     * @return bool
     */
    public function hasLinks(): bool {
        return !empty($this->links);
    }

    /**
     * @return array
     */
    public function toArray(): array {
        $asArray = [
            'name'  => $this->name,
            'alias' => $this->alias,
            'order' => $this->order,
            'icon'  => $this->icon,
            'items' => []
        ];

        foreach ($this->getLinks() as $link) {
            $asArray['items'][] = $link->toArray();
        }

        return $asArray;
    }

    /**
     * @return array
     */
    public function getLinks(): array {
        return $this->links;
    }

}
