<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

namespace itnovum\openITCOCKPIT\Core\ValueObjects;

use Authentication\IdentityInterface;
use itnovum\openITCOCKPIT\Core\Views\UserTime;

class User {
    /**
     * @var IdentityInterface
     */
    private $Identity;

    /**
     * @var bool
     */
    private $recursiveBrowser;

    /**
     * @var mixed|null
     */
    private $fullName;

    /**
     * @var int
     */
    private $id;

    /**
     * @var string
     */
    private $timezone;

    /**
     * @var string
     */
    private $dateformat;

    /**
     * User constructor.
     * @param IdentityInterface $Identity
     */
    public function __construct(IdentityInterface $Identity) {
        $this->Identity = $Identity;

        $this->recursiveBrowser = (bool)$Identity->get('recursive_browser');
        $this->fullName = sprintf('%s %s', $Identity->get('firstname'), $Identity->get('lastname'));
        $this->id = (int)$Identity->get('id');
        $this->timezone = $Identity->get('timezone');
        $this->dateformat = $Identity->get('dateformat');

    }

    /**getUserTime
     * @return boolean
     */
    public function isRecursiveBrowserEnabled() {
        return $this->recursiveBrowser;
    }

    /**
     * @return mixed|null
     */
    public function getFullName() {
        return $this->fullName;
    }

    /**
     * @return int
     */
    public function getId() {
        return $this->id;
    }

    /**
     * @return string
     */
    public function getTimezone() {
        return $this->timezone;
    }

    /**
     * @return string
     */
    public function getDateformat() {
        return $this->dateformat;
    }

    /**
     * @return UserTime
     */
    public function getUserTime() {
        return new UserTime($this->timezone, $this->dateformat);
    }


}
