<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.
namespace itnovum\openITCOCKPIT\Core\Views;


use Cake\I18n\FrozenTime;

abstract class Acknowledgement {


    /**
     * @var int
     */
    private $acknowledgement_type;

    /**
     * @var string
     */
    private $author_name;

    /**
     * @var string
     */
    private $comment_data;

    /**
     * @var int|string
     */
    private $entry_time;


    /**
     * @var bool
     */
    private $is_sticky;

    /**
     * @var bool
     */
    private $notify_contacts;

    /**
     * @var bool
     */
    private $persistent_comment;

    /**
     * @var int
     */
    private $state;

    /**
     * @var UserTime|null
     */
    private $UserTime;

    /**
     * StatehistoryHost constructor.
     * @param array $data
     */
    public function __construct($data, $UserTime = null) {
        if (isset($data['acknowledgement_type'])) {
            $this->acknowledgement_type = (int)$data['acknowledgement_type'];
        }

        if (isset($data['author_name'])) {
            $this->author_name = $data['author_name'];
        }

        if (isset($data['comment_data'])) {
            $this->comment_data = $data['comment_data'];
        }

        if (isset($data['entry_time'])) {
            $this->entry_time = $data['entry_time'];
        }

        if (isset($data['is_sticky'])) {
            $this->is_sticky = (bool)$data['is_sticky'];
        }

        if (isset($data['notify_contacts'])) {
            $this->notify_contacts = (bool)$data['notify_contacts'];
        }

        if (isset($data['persistent_comment'])) {
            $this->persistent_comment = (bool)$data['persistent_comment'];
        }

        if (isset($data['state'])) {
            $this->state = (int)$data['state'];
        }

        $this->UserTime = $UserTime;
    }

    /**
     * @return int
     */
    public function getAcknowledgementType() {
        return $this->acknowledgement_type;
    }

    /**
     * @return string
     */
    public function getAuthorName() {
        return $this->author_name;
    }

    /**
     * @return string
     */
    public function getCommentData() {
        return $this->comment_data;
    }

    /**
     * @return int|string
     */
    public function getEntryTime() {
        if (!is_numeric($this->entry_time)) {
            if ($this->entry_time instanceof FrozenTime) {
                $this->entry_time = $this->entry_time->timestamp;
            } else {
                $this->entry_time = strtotime($this->entry_time);
            }
        }

        return $this->entry_time;
    }

    /**
     * @return boolean
     */
    public function isSticky() {
        return $this->is_sticky;
    }

    /**
     * @return boolean
     */
    public function hasNotifyContacts() {
        return $this->notify_contacts;
    }

    /**
     * @return boolean
     */
    public function isPersistentComment() {
        return $this->persistent_comment;
    }

    /**
     * @return int
     */
    public function getState() {
        return $this->state;
    }

    /**
     * @return array
     */
    public function toArray() {
        $arr = get_object_vars($this);
        if (isset($arr['UserTime'])) {
            unset($arr['UserTime']);
        }

        if ($this->UserTime !== null) {
            $arr['entry_time'] = $this->UserTime->format($this->getEntryTime());
        } else {
            $arr['entry_time'] = $this->getEntryTime();
        }

        return $arr;
    }
}
