var openITCOCKPIT = angular.module('openITCOCKPIT', ['ui.router'])
    .factory("httpInterceptor", function($q, $rootScope, $timeout, $state){
        return {
            response: function(result){
                if(result.data.hasOwnProperty('_csrfToken')){
                    if(result.data._csrfToken !== null){
                        $rootScope._csrfToken = result.data._csrfToken;
                    }
                }

                $('#global_ajax_loader').fadeOut('slow');
                return result || $.then(result)
            },
            request: function(response){
                if(response.method !== 'GET'){
                    response.headers['X-CSRF-Token'] = $rootScope._csrfToken;
                }

                $('#global_ajax_loader').show();
                return response || $q.when(response);
            },
            responseError: function(rejection){
                $('#global_ajax_loader').fadeOut('slow');

                if(rejection.status === 401){
                    window.location = '/users/login';
                }

                if(rejection.status === 403){
                    $state.go('403');
                }

                if(rejection.status === 404){
                    $state.go('404');
                }

                return $q.reject(rejection);
            }
        };
    })

    .config(function($httpProvider){
        $httpProvider.interceptors.push("httpInterceptor");
    })

    .config(function($urlRouterProvider, $stateProvider){
        $urlRouterProvider.otherwise("/dashboards/index");

        $stateProvider

            .state('403', {
                url: '/error/403',
                templateUrl: "/angular/forbidden.html",
                controller: "Error403Controller"
            })

            .state('404', {
                url: '/error/404',
                templateUrl: "/angular/not_found.html",
                controller: "Error404Controller"
            })

            .state('ProfileEdit', {
                url: '/profile/edit',
                templateUrl: "/profile/edit.html",
                controller: "ProfileEditController"
            })

            .state('DashboardsIndex', {
                url: '/dashboards/index',
                templateUrl: "/dashboard/index.html",
                controller: "DashboardsIndexController"
            })

            .state('HostsIndex', {
                url: '/hosts/index/:statename',
                params: {
                    statename: {
                        value: null,
                        squash: true
                    },
                    hostname: {
                        value: null,
                        squash: true
                    }
                },
                templateUrl: "/hosts/index.html",
                controller: "HostsIndexController"
            })

            .state('HostsBrowser', {
                url: '/hosts/browser/:uuid',
                templateUrl: "/hosts/browser.html",
                controller: "HostsBrowserController"
            })

            .state('HostchecksIndex', {
                url: '/hostchecks/index/:uuid',
                templateUrl: "/hostchecks/index.html",
                controller: "HostchecksIndexController"
            })

            .state('StatehistoriesHost', {
                url: '/statehistories/host/:uuid',
                templateUrl: "/statehistories/host.html",
                controller: "StatehistoriesHostController"
            })

            .state('AcknowledgementsHost', {
                url: '/acknowledgements/host/:uuid',
                templateUrl: "/acknowledgements/host.html",
                controller: "AcknowledgementsHostController"
            })

            .state('DowntimesHost', {
                url: '/downtimes/host/:uuid',
                templateUrl: "/downtimes/host.html",
                controller: "DowntimesHostController"
            })

            .state('ServicesIndex', {
                url: '/services/index/:statename/:hostdisplayname',
                params: {
                    statename: {
                        value: null,
                        squash: true
                    },
                    hostdisplayname: {
                        value: null,
                        squash: true
                    },
                    servicename: {
                        value: null,
                        squash: true
                    }
                },
                templateUrl: "/services/index.html",
                controller: "ServicesIndexController"
            })

            .state('ServicesBrowser', {
                url: '/services/browser/:hostuuid/:serviceuuid',
                templateUrl: "/services/browser.html",
                controller: "ServicesBrowserController"
            })

            .state('ServicechecksIndex', {
                url: '/servicechecks/index/:serviceuuid',
                templateUrl: "/servicechecks/index.html",
                controller: "ServicechecksIndexController"
            })

            .state('StatehistoriesService', {
                url: '/statehistories/service/:serviceuuid',
                templateUrl: "/statehistories/service.html",
                controller: "StatehistoriesServiceController"
            })

            .state('AcknowledgementsService', {
                url: '/acknowledgements/service/:serviceuuid',
                templateUrl: "/acknowledgements/service.html",
                controller: "AcknowledgementsServiceController"
            })

            .state('DowntimesService', {
                url: '/downtimes/service/:serviceuuid',
                templateUrl: "/downtimes/service.html",
                controller: "DowntimesServiceController"
            })

            .state('AutomapsIndex', {
                url: '/automaps/index',
                templateUrl: "/services/automap.html",
                controller: "AutomapsIndexController"
            })

            .state('UsersIndex', {
                url: '/users/index',
                templateUrl: "/Users/index.html",
                controller: "UsersIndexController",
                params: {
                    usergroup_id: {
                        value: null,
                        array: true,
                        squash: false
                    },
                }
            })

            .state('UsersAdd', {
                url: '/users/add',
                templateUrl: "/Users/add.html",
                controller: "UsersAddController"
            })
            .state('UsersEdit', {
                url: '/users/edit/:id',
                templateUrl: "/Users/edit.html",
                controller: "UsersEditController"
            })

            .state('UsergroupsIndex', {
                url: '/usergroups/index',
                templateUrl: "/Usergroups/index.html",
                controller: "UsergroupsIndexController"
            })

            .state('UsergroupsAdd', {
                url: '/usergroups/add',
                templateUrl: "/Usergroups/add.html",
                controller: "UsergroupsAddController"
            })

            .state('UsergroupsEdit', {
                url: '/usergroups/edit/:id',
                templateUrl: "/Usergroups/edit.html",
                controller: "UsergroupsEditController"
            })

    })

    .filter('hostStatusName', function(){
        return function(hoststatusId){
            if(typeof hoststatusId === 'undefined'){
                return false;
            }

            switch(hoststatusId){
                case 0:
                    return 'Up';
                case 1:
                    return 'Down';
                default:
                    return 'Unreachable';
            }

        }
    })

    .filter('serviceStatusName', function(){
        return function(servicestatusId){
            if(typeof servicestatusId === 'undefined'){
                return false;
            }

            switch(servicestatusId){
                case 0:
                    return 'Ok';
                case 1:
                    return 'Warning';
                case 2:
                    return 'Critical';
                default:
                    return 'Unknown';
            }

        }
    })

    .filter('encodeURI', function(){
        return function(str){
            return encodeURI(str);
        }
    })

    .filter('highlight', function($sce){
        return function(title, searchString){
            searchString = searchString.replace(/\s/g, "");
            var newSearchString = "";
            for(var i = 0; i < searchString.length; i++){
                newSearchString += searchString.charAt(i) + "\\s*";
            }
            if(searchString) title = title.replace(new RegExp('(' + newSearchString + ')', 'gi'),
                '<span class="search-highlight">$1</span>');

            return $sce.trustAsHtml(title)
        }
    })

    .filter('trustAsHtml', function($sce){
        return function(text){
            return $sce.trustAsHtml(text);
        };
    })

    .run(function($rootScope, SortService){

        $rootScope.getSortClass = function(field){
            if(field === SortService.getSort()){
                if(SortService.getDirection() === 'asc'){
                    return 'fa-sort-asc';
                }
                return 'fa-sort-desc';
            }

            return 'fa-sort';
        };

        $rootScope.orderBy = function(field){
            if(field !== SortService.getSort()){
                SortService.setDirection('asc');
                SortService.setSort(field);
                SortService.triggerReload();
                return;
            }

            if(SortService.getDirection() === 'asc'){
                SortService.setDirection('desc');
            }else{
                SortService.setDirection('asc');
            }
            SortService.triggerReload();
        };

        $rootScope.currentStateForApi = function(current_state){
            var states = [];
            for(var key in current_state){
                if(current_state[key] === true || current_state[key] === 1){
                    states.push(key);
                }
            }
            return states;
        };
    })
;
