<?php
// Copyright (C) <2018>  <it-novum GmbH>
//
// This file is dual licensed
//
// 1.
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, version 3 of the License.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// 2.
//  If you purchased an openITCOCKPIT Enterprise Edition you can use this file
//  under the terms of the openITCOCKPIT Enterprise Edition license agreement.
//  License agreement and license key will be shipped with the order
//  confirmation.

namespace itnovum\openITCOCKPIT\Checks\Receiver\Plugins;


use itnovum\openITCOCKPIT\Checks\Receiver\NagiosService;
use itnovum\openITCOCKPIT\Checks\Receiver\ValueObjects\PerfdataGauge;

/**
 * Class Agent
 * @package itnovum\openITCOCKPIT\Checks\Receiver\Plugins
 */
class Agent extends BasePlugin implements PluginInterface {

    /**
     * @var float
     */
    private $responseTime;

    public function __construct($pluginConfig, $json, $responseTime = 0.0) {
        $this->responseTime = $responseTime;
        parent::__construct($pluginConfig, $json);
    }

    public function execute() {
        if ($this->checkError('agent')) {
            return;
        }

        if (!isset($this->json['agent'])) {
            $this->setOutput('Key "agent" not found in Agent output');
            $this->setStatuscode(NagiosService::STATUS_UNKNOWN);
            return;
        }

        $agentJson = $this->json['agent'];

        if (version_compare($agentJson['agent_version'], '3.0.0', '<')) {
            $this
                ->setOutput(sprintf(
                    'Agent version %s is EOL not supported anymore. Please [url=https://github.com/it-novum/openitcockpit-agent-go/releases]download the latest version[/url].',
                    $agentJson['agent_version']
                ))
                ->setStatuscode(NagiosService::STATUS_CRITICAL);
            return;
        }

        $this->addPerfdata(new PerfdataGauge(
            'response_time',
            $this->responseTime,
            's',
            15,
            30,
            0
        ));

        if ($agentJson['goos'] === 'windows') {
            $this
                ->setOutput(sprintf(
                    'Agent version %s running on %s (Release: %s, Build: %s). Last update: %s',
                    $agentJson['agent_version'],
                    $agentJson['system'],
                    $agentJson['windows_release_id'],
                    $agentJson['windows_current_build'],
                    date('Y-m-d H:i:s', $agentJson['last_updated_timestamp'])
                ))
                ->setStatuscode(NagiosService::STATUS_OK);
        } else {
            // Linux and darwin / macOS
            $this
                ->setOutput(sprintf(
                    'Agent version %s running on %s (%s). Last update: %s',
                    $agentJson['agent_version'],
                    $agentJson['system'],
                    $agentJson['mac_version'] ? $agentJson['mac_version'] : $agentJson['kernel_version'],
                    date('Y-m-d H:i:s', $agentJson['last_updated_timestamp'])
                ))
                ->setStatuscode(NagiosService::STATUS_OK);
        }

    }

}
