<?php
// Copyright (C) <2018>  <it-novum GmbH>
//
// This file is dual licensed
//
// 1.
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, version 3 of the License.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// 2.
//  If you purchased an openITCOCKPIT Enterprise Edition you can use this file
//  under the terms of the openITCOCKPIT Enterprise Edition license agreement.
//  License agreement and license key will be shipped with the order
//  confirmation.

namespace itnovum\openITCOCKPIT\Checks\Receiver\Plugins;


use itnovum\openITCOCKPIT\Checks\Receiver\NagiosService;
use itnovum\openITCOCKPIT\Checks\Receiver\ValueObjects\PerfdataGauge;

/**
 * Class Alfresco
 * @package itnovum\openITCOCKPIT\Checks\Receiver\Plugins
 * @deprecated
 */
class Alfresco extends BasePlugin implements PluginInterface {

    public function execute() {

        $requiredKeys = [
            'alfrescostats'
        ];

        foreach ($requiredKeys as $requiredKey) {
            if (!isset($this->json[$requiredKey])) {
                $this->setOutput(sprintf('Key "%s" not found in Agent output', $requiredKey));
                $this->setStatuscode(NagiosService::STATUS_UNKNOWN);
                return;
            }
        }

        if (!isset($this->pluginConfig['args'])) {
            $this->setOutput('Plugin arguments not found in configuration');
            $this->setStatuscode(NagiosService::STATUS_UNKNOWN);
            return;
        }

        $name = '';
        foreach ($this->pluginConfig['args'] as $key => $arg) {
            switch ($key) {
                case 0:
                    $amountWarning = $arg;
                    break;
                case 1:
                    $amountCritical = $arg;
                    break;
                case 2:
                    $name = $arg;
                    break;
            }
        }

        foreach ($this->json['alfrescostats'] as $i => $check) {
            if ($check['name'] === $name) {
                $shortname = explode('=', $name)[1];

                $this->addPerfdata(new PerfdataGauge(
                    str_replace('=', '/', $name),
                    $check['value'],
                    '',
                    $amountWarning,
                    $amountCritical,
                    0
                ));

                $preformat = '';
                $format = $name . ': %s';
                $this->setStatuscode(NagiosService::STATUS_OK);

                if ($this->getRangeStatus($amountWarning, $check['value'])) {
                    $preformat = $shortname . ' is warning! ';
                    $this->setStatuscode(NagiosService::STATUS_WARNING);
                }

                if ($this->getRangeStatus($amountCritical, $check['value'])) {
                    $preformat = $shortname . ' is critical! ';
                    $this->setStatuscode(NagiosService::STATUS_CRITICAL);
                }

                $this->setOutput(sprintf(
                    $preformat . $format,
                    $check['value']
                ));
                return;
            }
        }

        $this->setOutput(sprintf('No alfresco check result found matching name "%s"!', $name));
        $this->setStatuscode(NagiosService::STATUS_UNKNOWN);
        return;
    }
}
