<?php
// Copyright (C) <2018>  <it-novum GmbH>
//
// This file is dual licensed
//
// 1.
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, version 3 of the License.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// 2.
//  If you purchased an openITCOCKPIT Enterprise Edition you can use this file
//  under the terms of the openITCOCKPIT Enterprise Edition license agreement.
//  License agreement and license key will be shipped with the order
//  confirmation.

namespace itnovum\openITCOCKPIT\Checks\Receiver\Plugins;


use itnovum\openITCOCKPIT\Checks\Receiver\Converter\Byte;
use itnovum\openITCOCKPIT\Checks\Receiver\NagiosService;
use itnovum\openITCOCKPIT\Checks\Receiver\ValueObjects\PerfdataGauge;

/**
 * Class MemoryUsage
 * @package itnovum\openITCOCKPIT\Checks\Receiver\Plugins
 */
class MemoryUsage extends BasePlugin implements PluginInterface {

    public function execute() {
        if ($this->checkError('memory')) {
            return;
        }

        $requiredKeys = [
            'memory' => [
                'total',
                'available',
                'percent',
                'used',
                'free'
            ]
        ];

        foreach ($requiredKeys as $mainKey => $subKeys) {
            foreach ($subKeys as $subKey) {
                if (!isset($this->json[$mainKey][$subKey])) {
                    $this->setOutput(sprintf('Key "%s.%s" not found in Agent output', $mainKey, $subKey));
                    $this->setStatuscode(NagiosService::STATUS_UNKNOWN);
                    return;
                }
            }
        }

        if (!isset($this->pluginConfig['args'])) {
            $this->setOutput('Plugin arguments not found in configuration');
            $this->setStatuscode(NagiosService::STATUS_UNKNOWN);
            return;
        }

        foreach ($this->pluginConfig['args'] as $key => $arg) {
            switch ($key) {
                case 0:
                    $warning = $arg;
                    break;
                case 1:
                    $critical = $arg;
                    break;
                case 2:
                    $unit = $arg;
                    break;
            }
        }

        if ($unit != '%' && !in_array($unit, Byte::supportedUnits(), true)) {
            $this->setOutput('Invalid argument format');
            $this->setStatuscode(NagiosService::STATUS_UNKNOWN);
            return;
        }

        $this->addPerfdata(new PerfdataGauge(
            'Memory_Used',
            round($this->json['memory']['percent'], 2),
            '%',
            $warning,
            $critical,
            0,
            100
        ));

        foreach (['total', 'available', 'used', 'free'] as $gauge) {
            if (isset($this->json['memory'][$gauge])) {
                $this->addPerfdata(new PerfdataGauge(
                    $gauge,
                    round($this->json['memory'][$gauge], 2),
                    'B'
                ));
            }
        }

        $output = sprintf(
            'Memory usage: %s/%s (%s%%).',
            Byte::asHuman($this->json['memory']['used']),
            Byte::asHuman($this->json['memory']['total']),
            round($this->json['memory']['percent'], 2)
        );

        foreach (['total', 'available', 'used', 'free'] as $gauge) {
            if (isset($this->json['memory'][$gauge])) {
                $output = sprintf(
                    '%s %s: %s,',
                    $output,
                    ucfirst($gauge),
                    Byte::asHuman($this->json['memory'][$gauge])
                );
            }
        }

        $this->setOutput($output);

        if ($unit == '%') {
            if ($this->getRangeStatus($critical, $this->json['memory']['percent'])) {
                $this->setStatuscode(NagiosService::STATUS_CRITICAL);
                return;
            }

            if ($this->getRangeStatus($warning, $this->json['memory']['percent'])) {
                $this->setStatuscode(NagiosService::STATUS_WARNING);
                return;
            }
        } else {
            $warning = Byte::scaleBytesToUnit($warning, $unit);
            $critical = Byte::scaleBytesToUnit($critical, $unit);
            $used = Byte::scaleBytesToUnit($this->json['memory']['used'], $unit);


            if ($this->getRangeStatus($critical, $used)) {
                $this->setStatuscode(NagiosService::STATUS_CRITICAL);
                return;
            }

            if ($this->getRangeStatus($warning, $used)) {
                $this->setStatuscode(NagiosService::STATUS_WARNING);
                return;
            }
        }

        $this->setStatuscode(NagiosService::STATUS_OK);
    }

}
