<?php
// Copyright (C) <2018>  <it-novum GmbH>
//
// This file is dual licensed
//
// 1.
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, version 3 of the License.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// 2.
//  If you purchased an openITCOCKPIT Enterprise Edition you can use this file
//  under the terms of the openITCOCKPIT Enterprise Edition license agreement.
//  License agreement and license key will be shipped with the order
//  confirmation.

namespace itnovum\openITCOCKPIT\Checks\Receiver\Plugins;


use itnovum\openITCOCKPIT\Checks\Receiver\NagiosService;
use itnovum\openITCOCKPIT\Checks\Receiver\ValueObjects\PerfdataGauge;

/**
 * Class SystemLoad
 * @package itnovum\openITCOCKPIT\Checks\Receiver\Plugins
 */
class SystemLoad extends BasePlugin implements PluginInterface {

    public function execute() {
        if ($this->checkError('system_load')) {
            return;
        }

        $requiredKeys = [
            'system_load'
        ];

        foreach ($requiredKeys as $requiredKey) {
            if (!isset($this->json[$requiredKey])) {
                $this->setOutput(sprintf('Key "%s" not found in Agent output', $requiredKey));
                $this->setStatuscode(NagiosService::STATUS_UNKNOWN);
                return;
            }
        }

        if (!isset($this->pluginConfig['args'])) {
            $this->setOutput('Plugin arguments not found in configuration');
            $this->setStatuscode(NagiosService::STATUS_UNKNOWN);
            return;
        }

        $timerange = 1;
        $load_to_check = 0;
        $warning = 24;
        $critical = 32;
        foreach ($this->pluginConfig['args'] as $key => $arg) {
            switch ($key) {
                case 0:
                    $warning = floatval($arg);
                    break;
                case 1:
                    $critical = floatval($arg);
                    break;
                case 2:
                    $timerange = intval($arg);
                    break;
            }
        }

        switch ($timerange) {
            case 5:
                // use load5
                $load_to_check = round($this->json['system_load'][1], 2);
                break;
            case 5:
                // use load15
                $load_to_check = round($this->json['system_load'][2], 2);
                break;
            default:
                // use load1
                $load_to_check = round($this->json['system_load'][0], 2);
                break;
        }


        $this->addPerfdata(new PerfdataGauge(
            'load1',
            round($this->json['system_load'][0], 2),
            '',
            $warning,
            $critical,
            0
        ));

        $this->addPerfdata(new PerfdataGauge(
            'load5',
            round($this->json['system_load'][1], 2),
            '',
            $warning,
            $critical,
            0
        ));

        $this->addPerfdata(new PerfdataGauge(
            'load15',
            round($this->json['system_load'][2], 2),
            '',
            $warning,
            $critical,
            0
        ));


        $this->setOutput(sprintf(
            'Average system load: %s %s %s',
            round($this->json['system_load'][0], 2),
            round($this->json['system_load'][1], 2),
            round($this->json['system_load'][2], 2)
        ));


        if ($this->getRangeStatus($critical, $load_to_check)) {
            $this->setStatuscode(NagiosService::STATUS_CRITICAL);
            return;
        }

        if ($this->getRangeStatus($warning, $load_to_check)) {
            $this->setStatuscode(NagiosService::STATUS_WARNING);
            return;
        }

        $this->setStatuscode(NagiosService::STATUS_OK);
    }

}
