<?php
// Copyright (C) <2018>  <it-novum GmbH>
//
// This file is dual licensed
//
// 1.
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, version 3 of the License.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// 2.
//  If you purchased an openITCOCKPIT Enterprise Edition you can use this file
//  under the terms of the openITCOCKPIT Enterprise Edition license agreement.
//  License agreement and license key will be shipped with the order
//  confirmation.

namespace itnovum\openITCOCKPIT\Checks\Receiver\Plugins;


use itnovum\openITCOCKPIT\Checks\Receiver\NagiosService;
use itnovum\openITCOCKPIT\Checks\Receiver\ValueObjects\PerfdataGauge;

/**
 * Class Temperature
 * @package itnovum\openITCOCKPIT\Checks\Receiver\Plugins
 */
class Temperature extends BasePlugin implements PluginInterface {

    public function execute() {
        if ($this->checkError('sensors')) {
            return;
        }
        if ($this->checkError('sensors', 'Batteries')) {
            return;
        }

        $requiredKeys = [
            'sensors' => [
                'Batteries'
            ]
        ];

        foreach ($requiredKeys as $mainKey => $subKeys) {
            foreach ($subKeys as $subKey) {
                if (!isset($this->json[$mainKey][$subKey])) {
                    $this->setOutput(sprintf('Key "%s.%s" not found in Agent output', $mainKey, $subKey));
                    $this->setStatuscode(NagiosService::STATUS_UNKNOWN);
                    return;
                }
            }
        }

        if (!isset($this->pluginConfig['args'])) {
            $this->setOutput('Plugin arguments not found in configuration');
            $this->setStatuscode(NagiosService::STATUS_UNKNOWN);
            return;
        }

        foreach ($this->pluginConfig['args'] as $key => $arg) {
            switch ($key) {
                case 0:
                    $warning = $arg;
                    break;
                case 1:
                    $critical = $arg;
                    break;
                case 2:
                    $label = trim($arg);
                    break;
            }
        }

        foreach ($this->json['sensors']['Temperatures'] as $sensor) {
            if ($sensor['label'] === $label) {
                $this->setOutput(sprintf(
                    'Current %s temperature is %s°C',
                    $sensor['label'],
                    $sensor['current']
                ));

                $this->addPerfdata(new PerfdataGauge(
                    'current',
                    $sensor['current'],
                    'C',
                    $warning,
                    $critical
                ));

                $this->setStatuscode(NagiosService::STATUS_OK);

                if ($this->getRangeStatus($critical, $sensor['current'])) {
                    $this->setStatuscode(NagiosService::STATUS_CRITICAL);
                    return;
                }

                if ($this->getRangeStatus($warning, $sensor['current'])) {
                    $this->setStatuscode(NagiosService::STATUS_WARNING);
                    return;
                }
                return;
            }
        }

        $this->setOutput(sprintf('Sensor "%s" not found!', $label));
        $this->setStatuscode(NagiosService::STATUS_UNKNOWN);

    }
}
